# -*- coding: utf-8 -*-
"""The base for all classes modeling hardware."""
from collections import OrderedDict
from typing import List, Union

from micado.main.MObject import MObject


def get_attr_dictionary(attr: Union[MObject, List[MObject]]):
    """Get ScopeSIM dictionary for attr.

    Args:
        attr: Value of an persistent attribute.
    """
    from micado.main.Hardware import Hardware

    if isinstance(attr, Hardware):
        d = attr.scopesim_dictionary
    elif isinstance(attr, list):
        # TODO: this only works for lists of Hardware, not lists of values
        if len(attr):
            lds = [attri.scopesim_dictionary for attri in attr]
            lds0a = lds[0]["kwargs"]
            # Create array_dicts from this array of dictionaries.
            da = OrderedDict(
                [
                    (k, [d2["kwargs"][k] for d2 in lds])
                    for k in lds0a
                    if not k.endswith("_unit")
                ]
            )
            # But keep the units separate.
            # The are always identical, since they are hardcoded in the model.
            dunits = OrderedDict(
                [(k, v) for k, v in lds0a.items() if k.endswith("_unit")]
            )

            kwargs2 = {"array_dict": da}
            kwargs2.update(dunits)

            d = lds[0]
            d["class"] += "List"
            d["kwargs"] = kwargs2
        else:
            # TODO: test this
            d = None
    else:
        d = attr

    return d


class HardwareExtra(MObject):
    """The base for all classes modeling hardware."""

    def scopesimname_from_attrname(self, attrname):
        """Return ScopeSim name from attrname."""
        # TODO: Move this to FilterCurveExtra.
        # TODO: Make this just name in ScopeSIM.
        # from micado.hardware.FilterCurve import FilterCurve
        from micado.hardware.FilterWheel import FilterWheel

        # if isinstance(self, FilterCurve) and attrname == "name":
        #     return "filter_name"
        if isinstance(self, FilterWheel) and attrname == "name":
            return "current_filter"
        return attrname

    @property
    def scopesim_dictionary(self):
        """Create dictionary for ScopeSIM."""
        attrnames = [
            attrname
            for attrname in self.get_persistent()
            if not attrname.startswith("dummy") and attrname not in ["object_id"]
        ]
        dict_attr = OrderedDict(
            [
                (
                    self.scopesimname_from_attrname(attrname),
                    get_attr_dictionary(getattr(self, attrname)),
                )
                for attrname in attrnames
            ]
            + [
                (
                    f"{attrname}_unit",
                    getattr(self.__class__, attrname).__doc__.split("[")[-1].strip("]"),
                )
                for attrname in attrnames
            ]
        )
        dict_attr_no_none_unit = OrderedDict(
            [
                (k, v)
                for k, v in dict_attr.items()
                if not (k.endswith("_unit") and v == "None")
            ]
        )

        dd = {
            "class": self.__class__.__name__,
            "description": self.__doc__.strip(),
            "kwargs": dict_attr_no_none_unit,
        }

        return dd
