# -*- coding: utf-8 -*-
"""Tests for MasterFlat.py."""

from micado.config.Environment import Env
from micado.dataitems.MasterFlat import MasterFlat
from micado.dataitems.MasterFlatExtension import MasterFlatExtension
from micado.dataitems.DepersistedFlat import DepersistedFlat
from micado.hardware.Detector import Detector
from micado.main.Imstat import Imstat
from micado.recipes.McdImgFlat import McdImgFlat
from micado.test.functional.dark_test import remove_file_and_links
from micado.test.functional.flat_test import generate_flat
from micado.test.functional.masterdark_test import (
    mock_depersist,
    compare_pros,
    create_masterdark,
)


def create_masterflat(simple: bool = False):
    """Create MasterFlat for testing.

    Args:
        simple: Generate a very small MasterFlat from 1 raw Flat.
    """
    # Create a MasterFlat from 1 Flat of 16x16 pixels when simple is set.
    n_flats = 5 if not simple else 1
    # TODO: At some point set this back to 4096
    size = 32

    mydetector = Detector(
        x_size=size,
        y_size=size,
    )
    mydetectorlist = [mydetector]

    dark = 2.0

    # Create raw Darks.
    myflats = []
    mydepersistedflats = []
    for _ in range(n_flats):
        myflat = generate_flat(dark=dark, size=size)
        myflat.make_simulation()
        myflat.save_fits()
        mydepersistedflat = mock_depersist(DepersistedFlat, myflat)
        mydepersistedflat.save_fits()

        myflats.append(myflat)
        mydepersistedflats.append(mydepersistedflat)

    mydepersisteddarks, mymasterdark, _, mydarks = create_masterdark()
    mydarks = mydepersisteddarks + [mymasterdark] + mydarks

    myrecipe = McdImgFlat()

    mymasterextensions = [
        MasterFlatExtension(BITPIX=-64, NAXIS=2) for _ in mydetectorlist
    ]

    mymasterflat = MasterFlat(
        recipe=myrecipe,
        flats=mydepersistedflats,
        masterdark=mymasterdark,
        extensions=mymasterextensions,
    )
    mymasterflat.make_proto()
    mymasterflat.save_fits()
    return mydepersistedflats, mymasterflat, myflats, mydarks


def test_processing():
    """Test creation of MasterFlat."""
    Env["use_python_logging"] = True
    myflats, mymasterflat, myflats_raw, mydarks = create_masterflat()

    # Create image statistics.
    myimstatraw = Imstat(baseframe=myflats[0])
    myimstatraw.make()

    myimstat = Imstat(baseframe=mymasterflat)
    myimstat.make()

    # The standard deviation cannot be used to measure the
    # effectiveness of the MasterFlat routine, because the
    # structure of the flat means that the noise does not
    # scale with sqrt(n_raws) over the full image.
    # TODO: assert myimstat.std < myimstatraw.std / 1.5

    # TODO: Verify some actual requirement.

    fn = mymasterflat.filename

    master2 = MasterFlat(filename=fn)
    compare_pros(mymasterflat, master2)

    for dm in myflats + myflats_raw + mydarks:
        remove_file_and_links(dm.filename)

    remove_file_and_links(mymasterflat.filename)
