# -*- coding: utf-8 -*-
"""Create TikZ figures from svg."""
import io
import os
import re
from typing import Tuple

from svg2tikz.extensions.tikz_export import TikZPathExporter
from svg2tikz.inkexlib import inkex

from micado.toolbox.ditaa_from_wise import PATH_DOCS_GENERATED


def get_link_from_term(term, black=False, association=False):
    r"""Create link from term.

    E.g.
    r"\hyperref[dataitem:DEPERSISTEDDARKIMG]{\PROD{DEPERSISTEDDARK_IMG}}"
    """
    white = "WHITE" if not black else ""
    assoc = "ASSOC" if association else ""
    term_link = term.replace("\\_", "").replace("[", "").replace("]", "")
    if term[-4:] in ["_CAT", "_RAW", "_EXT", "_TAB", "_IMG", "_HDR"]:
        # Dataitem. \PROD
        return rf"\hyperref[dataitem:{term_link}]{{\{white}{assoc}FITS{{{term}}}}}"
    elif term.upper() == term:
        # Process parameter, do nothing for now.
        return rf"\FITS{{{term}}}"
    elif term.startswith("mcd\\_") or term.startswith("micado\\_"):
        # Recipe. \REC
        # TODO: DRL functions should also start with micado_ so this collides
        if association:
            return rf"\hyperref[recipe:{term_link}]{{\ASSOCREC{{{term}}}}}"
        else:
            return rf"\hyperref[recipe:{term_link}]{{\FITS{{{term}}}}}"
    elif (
        term.startswith("MCD\\_")
        or term.startswith("MICINSNAME")
        or term.startswith("MICADO")
    ):
        # Template.
        return rf"\hyperref[template:{term_link}]{{\FITS{{{term}}}}}"
    else:
        # RecipeStep / DRL Function. \REC
        return rf"\hyperref[dataitem:{term_link}]{{\{white}FITS{{{term}}}}}"


def get_plain_from_term(term, black=False, association=False):
    r"""Create plain from term.

    E.g. r"\PROD{DEPERSISTEDDARK_IMG}}"
    """
    white = "WHITE" if not black else ""
    if term[-4:] in ["_CAT", "_RAW", "_EXT", "_TAB", "_IMG", "_HDR"]:
        # Dataitem. \PROD
        return rf"\{white}FITS{{{term}}}"
    elif term.upper() == term:
        # Process parameter, do nothing for now.
        return rf"\FITS{{{term}}}"
    elif term.startswith("mcd\\_"):
        # Recipe. \REC
        return rf"\FITS{{{term}}}"
    elif term.startswith("MCD\\_") or term.startswith("MICINSNAME"):
        # Template.
        return rf"\FITS{{{term}}}"
    else:
        # RecipeStep / DRL Function. \REC
        return rf"\{white}FITS{{{term}}}"


def remove_whitefits(code_links):
    """Hack to remove white fits links.

    These only appear at the left side of the figures.
    """
    #   \path[fill=black] (46.0000,530.0000) node[above right] () {\hyperref[dataitem:PERSISTENCEIMG]{\WHITEFITS{PERSISTENCE\_IMG}}};
    badwhitefits = list(
        re.findall(r"(.*?\((\d+)\.\d*,\d+\.\d*\).*WHITEASSOCFITS.*;)", code_links)
    )
    for sbad, sxpos in badwhitefits:
        xpos = int(sxpos)
        if xpos < 150:
            sgood = sbad.replace("WHITEASSOCFITS", "ASSOCFITS")
            code_links = code_links.replace(sbad, sgood)
    return code_links


def tikz_from_svg(s_svg: str, inheritance=False, association=False) -> Tuple[str, str]:
    """Convert svg string to tikz string.

    inheritance means it is the inheritance matrix that is converted.
    """
    # Convert s_svg string to xml.
    stream = io.BytesIO(bytes(s_svg.encode("utf-8")))
    document = inkex.etree.parse(stream)
    # Setup svg2tikz, this is complicated because
    # 1) there seems to be a bug in svg2tikz when using strings, and
    # 2) it is necessary to fake command line options.
    effect = TikZPathExporter(inkscape_mode=False)
    effect.document = document
    # Scale of 0.8 is an heuristic.
    if inheritance:
        effect.getoptions(args=["--figonly", "--scale", "0.4"])
    elif association:
        effect.getoptions(args=["--figonly", "--scale", "0.8"])
    else:
        effect.getoptions(args=["--figonly", "--scale", "0.7"])
    effect.getselected()
    effect.getdocids()
    effect.effect()
    # Scale the text in the nodes as well by adding
    # every node/.style={scale=\globalscale}
    code2 = effect.output_code.replace(
        "outer sep=0pt]", r"outer sep=0pt, every node/.style={scale=\globalscale}]"
    )
    if inheritance:
        # The text should not be right of the given location, but centered.
        code2 = code2.replace(" right]", "]")
    # Find all MICADO related terms, being the terms with an underscore.
    micadoterms1 = re.findall(r"[\w\\_]+\\_[\w[\]]+", code2)
    # Remove 'xx', those are used in legend of association matrix.
    # Sort to do MCD\_img\_sci\_object after MCD\_img\_sci\_object\_base
    micadoterms = sorted(
        [t for t in set(micadoterms1) if "xx" not in t], key=lambda t: -len(t)
    )

    # Replace them with placeholders.
    for i, term in enumerate(micadoterms):
        code2 = code2.replace(term, f"TERMTOREPLACE{i}ECALPEROTMRET")

    # Replace them with links.
    code_links = code2
    for i, term in enumerate(micadoterms):
        code_links = code_links.replace(
            f"TERMTOREPLACE{i}ECALPEROTMRET",
            get_link_from_term(term, black=inheritance, association=association),
        )

    # Replace with plain text for calibration plan.
    for i, term in enumerate(micadoterms):
        code2 = code2.replace(
            f"TERMTOREPLACE{i}ECALPEROTMRET",
            get_plain_from_term(term, black=inheritance),
        )

    if association:
        code_links = remove_whitefits(code_links)

    return code2, code_links


def patch_association_matrix_scale(s_tikz):
    """Patch the scale of the association matrices.

    They won't fit on the page otherwise.
    """
    s_tikz = s_tikz.replace(
        r"\def \globalscale {0.800000}", r"\def \globalscale {0.250000}"
    )
    return s_tikz.replace(
        r"node/.style={scale=\globalscale}", r"node/.style={scale=0.33}"
    )


def main():
    """Loop through all svg files and create tikz images."""
    fns_svg = PATH_DOCS_GENERATED.glob("*.svg")
    for fn in fns_svg:
        fn_tikz = fn.with_name(fn.stem + "_tikz").with_suffix(".tex")
        print("tikz_from_svg svg ", fn)
        print("tikz_from_svg tikz", fn_tikz)
        # Remove the tikz file in order to prevent a PermissionError in the CI.
        os.remove(fn_tikz)
        with open(fn) as f:
            s_svg = f.read()
        _s_tikz_not_used, s_tikz_links = tikz_from_svg(
            s_svg,
            inheritance="inheritance" in fn.stem,
            association="association" in fn.stem,
        )
        if fn.name.startswith("associationmatrix"):
            s_tikz_links = patch_association_matrix_scale(s_tikz_links)

        # Use tikz with hyperlinks for DRLD.
        with open(fn_tikz, "w") as f:
            f.write(s_tikz_links)


if __name__ == "__main__":
    main()
