"""The Micado specific Pipelines"""

import os

from common.config.Environment import Env
from common.util.Pipeline import (
    Pipeline as Pipeline_common,
    PipelineError,
)

from micado.toolbox.node_overview import NodeInfoJob

from .SimulateRawJob import SimulateRawDarkJob, SimulateRawFlatJob

# DPU info for compute cluster peregrine
micado_peregrine = (
    "f119832@peregrine.hpc.rug.nl",  # remote_user
    "/home/f119832/dpu-run",  # remote_dir
    "/home/f119832/awehome/develop",  # awepipe
    "$TMPDIR",  # scratch
    "/home/f119832/awehome/Linux-centos-7.9-x86_64/develop/micado/bin/awe3.10",
    # awe_executable
)


class Pipeline(Pipeline_common):

    # the AWETARGET
    awetarget = "micado"

    # The default process time per dpu
    dpu_time_dict = {
        "dpu.hpc.rug.astro-wise.org": 1799,
    }

    # define rusert for these dpu's
    dpu_ruserts = {
        "dpu.hpc.rug.astro-wise.org": micado_peregrine,
        "dpu-test.hpc.rug.astro-wise.org": micado_peregrine,
    }

    def __init__(self):
        """initialize, set dpu_rusert if needed"""
        super().__init__()
        if Env["dpu_name"] in self.dpu_ruserts:
            self.dpu_rusert = self.dpu_ruserts[Env["dpu_name"]]

    @classmethod
    def create_pipeline(cls, sequence_identifier):
        """Factory function for Pipelines; returns an instance of the Pipeline
        which contains the specified sequence identifier"""
        if not cls.sequence_identifier_exists(sequence_identifier):
            raise PipelineError("Unkown sequence identifier %s" % sequence_identifier)
        for pipeline in cls.get_all_pipelines():
            if pipeline.has_sequence_identifier(sequence_identifier):
                return pipeline()
        raise PipelineError(
            "Could not instantiate a pipeline from the sequence identifier %s"
            % sequence_identifier
        )

    @classmethod
    def get_available_sequence_identifiers(cls):
        sequence_identifier_list = []
        for pipeline in cls.get_all_pipelines():
            sequence_identifier_list.extend(pipeline.sequence_identifiers())
        return sequence_identifier_list

    @classmethod
    def sequence_identifier_exists(cls, sequence_identifier):
        for pipeline in cls.get_all_pipelines():
            if pipeline.has_sequence_identifier(sequence_identifier):
                return True
        return False

    @classmethod
    def get_all_pipelines(cls):
        """return all the Pipeline classes"""
        return NodeInfo, SimulateRawDark, SimulateRawFlat

    @classmethod
    def create_paralleloptions(cls, pipeline, options):
        """Factory function for ParallelOptions classes"""
        return ParallelOptions(options)


class ParallelOptions(object):
    """A dictionary of options is duplicated in various ways (group filenames
    to be sent to different nodes, add a ifu clause etc.). This is handled
    by this set of classes."""

    def __init__(self, options):
        """initialize"""
        self.options = options

    def get_parallel_options(self):
        """return for each parallel job the options"""
        return [self.options]


class NodeInfo(Pipeline):
    task_sequences = {
        "nodeinfo": ["nodeinfo"],
        "nodeinfos": ["nodeinfo", "nodeinfos"],
    }
    atomic_task_map = {
        "nodeinfo": (NodeInfoJob, "SEQ"),
        "nodeinfos": (NodeInfoJob, "QUEUEWAIT"),
    }
    single_node_tasks = ["nodeinfos"]


class SimulateRawDark(Pipeline):
    """Simulate a raw Dark."""

    task_sequences = {
        "rawdark": ["rawdark"],
    }
    atomic_task_map = {
        "rawdark": (SimulateRawDarkJob, "SEQ"),
    }


class SimulateRawFlat(Pipeline):
    """Simulate a raw Flat."""

    task_sequences = {
        "rawflat": ["rawflat"],
    }
    atomic_task_map = {
        "rawflat": (SimulateRawFlatJob, "SEQ"),
    }
