"""
    Module with job definition for simulating raw data

    These are all the Raw classes, that have no derived classes:
         <class 'micado.dataitems.Dark.Dark'>: 1,
         <class 'micado.dataitems.Flat.Flat'>: 1,
         <class 'micado.dataitems.Nonlin.Nonlin'>: 1,
         <class 'micado.dataitems.Bkgfield.Bkgfield'>: 1,
         <class 'micado.dataitems.Illum.Illum'>: 1,
         <class 'micado.dataitems.CoroOffset.CoroOffset'>: 1,
         <class 'micado.dataitems.Astromfield.Astromfield'>: 1,
         <class 'micado.dataitems.Wam.Wam'>: 1,
         <class 'micado.dataitems.ScienceCoroWaffle.ScienceCoroWaffle'>: 1,
         <class 'micado.dataitems.ScienceCoroPupil.ScienceCoroPupil'>: 1,
         <class 'micado.dataitems.Science.Science'>: 1,
         <class 'micado.dataitems.ScienceAi.ScienceAi'>: 1,
         <class 'micado.dataitems.Stdfield.Stdfield'>: 1,
         <class 'micado.dataitems.Cam.Cam'>: 1,
         <class 'micado.dataitems.CoroBackground.CoroBackground'>: 1,
         <class 'micado.dataitems.DpsicsOpt.DpsicsOpt'>: 1

    These are all the Raw classes, that have 1 or more derived classes:
         <class 'micado.main.Raw.Raw'>: 25,
         <class 'micado.main.Opt.Opt'>: 23,
         <class 'micado.main.Sky.Sky'>: 17,
         <class 'micado.main.SkySiaiBase.SkySiaiBase'>: 3,
         <class 'micado.main.SkyHCI.SkyHCI'>: 9,
         <class 'micado.main.SkyHCIOnline.SkyHCIOnline'>: 3,
         <class 'micado.dataitems.CoroBase.CoroBase'>: 5,
         <class 'micado.dataitems.WamCamBase.WamCamBase'>: 3,
         <class 'micado.dataitems.DpsicsOptSciBase.DpsicsOptSciBase'>: 2,


    Start with these raw classes:

         <class 'micado.dataitems.Dark.Dark'>: 1,
         <class 'micado.dataitems.Flat.Flat'>: 1,
         <class 'micado.dataitems.Science.Science'>: 1,
         <class 'micado.dataitems.Stdfield.Stdfield'>: 1,
         <class 'micado.dataitems.Illum.Illum'>: 1,

    and optional:
         <class 'micado.dataitems.Bkgfield.Bkgfield'>: 1,
         <class 'micado.dataitems.Nonlin.Nonlin'>: 1,
"""

import copy

from datetime import datetime

from common.util.OptionsParser import OptionsParser

from micado.dataitems.Dark import Dark
from micado.dataitems.Flat import Flat
from micado.dataitems.DarkExtension import DarkExtension
from micado.dataitems.FlatExtension import FlatExtension
from micado.main.ObservingBlock import ObservingBlock
from micado.main.Raw import Raw
from micado.templates.MicadoImgCalDark import MicadoImgCalDark
from micado.templates.MicadoImgCalFlat import MicadoImgCalFlat
from micado.test.functional.detector_test import generate_detectorarray
from micado.test.functional.science_test import get_filt_inst_ro

from .Task import Task, TaskError, deserialize_object


class SimulateRawJobError(TaskError):
    """error raised in case of error"""

    pass


class SimulateRawJob(Task):
    """job runs simulate raw data"""

    # duration of job
    processing_time = 60  # seconds

    ## Defaults for these must be set in derived classes
    # the target to make
    target = Raw
    # default template class to use
    template_default = None
    # extension class to use
    extension = None
    # observing block target name
    observingblock_target = None

    raw_parser = OptionsParser(__name__)

    raw_parser.add_option(
        "c",
        "commit",
        False,
        "Commit switch, set True to commit results to database and data server",
    )
    raw_parser.add_option(
        "s",
        "skip_store",
        False,
        "Do not store on dataserver. Only to be used for testing.",
    )
    # parser.add_option("i", "instrument", "MICADO", "The name of the instrument")
    raw_parser.add_option(
        "t",
        "template",
        None,
        "The template; either give as object or as (class, object_id)",
    )
    raw_parser.add_option(
        "o",
        "observing_block",
        None,
        "The Observing Block; either give as object or as (class, object_id)",
    )
    raw_parser.add_option(
        "d",
        "detector",
        None,
        "The detector; either give as object or as (class, object_id)",
    )

    raw_parser.add_option(
        "mo",
        "mjdobs",
        "",
        "a measure for the start of the exposure, leave empty for value of tpl__start",
    )
    raw_parser.add_option(
        "ot", "obs__tplno", 0, "the number of the template in the observing block"
    )
    raw_parser.add_option(
        "te",
        "tpl__expno",
        target.tpl__expno.default,
        target.tpl__expno.__doc__,
    )
    raw_parser.add_option("ts", "tpl__start", "now", "the start of the template")

    def execute(self):
        """run the recipe"""
        args = self.init_arguments()
        obj = self.target(**args)
        obj.make()
        obj.save_fits()
        if self.commit:
            if not self.skip_store:
                obj.store()
            obj.commit()
        self.process_result = [obj.object_id.binary()]

    def init_arguments(self):
        """return arguments needed for __init__ of target"""
        # default now, if given query for existing ?
        if self.tpl__start == "now":
            tpl__start = datetime.now()
        else:
            tpl__start = datetime.fromisoformat(self.tpl__start)
        if self.mjdobs:
            mjdobs = datetime.fromisoformat(self.mjdobs)
        else:
            mjdobs = tpl__start
        # possible parameters for MicadoImgCalDark(det__dit=1, det__ndit=1)
        if self.template:
            template = deserialize_object(self.template)
        else:
            template = self.template_default()
        # configure name, target
        if self.observing_block:
            observing_block = deserialize_object(self.observing_block)
        else:
            # TODO also set name and instrument ?
            observing_block = ObservingBlock(
                target=self.observingblock_target,
                templates=[template],
            )
        # possible parameters for generate_detectorarray(dark=dark_real, size=size, n_det=1)
        if self.detector:
            detectorarray = deserialize_object(self.detector)
        else:
            detectorarray = generate_detectorarray()
        # define the extensions
        extensions = [
            self.extension(detector=det, BITPIX=-64, NAXIS=2)
            for det in detectorarray.detector_array_list
        ]
        return {
            "detector": detectorarray,
            "extensions": extensions,
            "mjdobs": mjdobs.timestamp(),
            "obs__tplno": self.obs__tplno,
            "observing_block": observing_block,
            "template": template,
            "tpl__expno": self.tpl__expno,
            "tpl__start": tpl__start.timestamp(),
        }


class SimulateOptJob(SimulateRawJob):
    """job runs simulate Raw Opt data"""

    opt_parser = copy.deepcopy(SimulateRawJob.raw_parser)
    opt_parser.name = __name__

    opt_parser.add_option(
        "i", "instrument", None, "ESO acronym for the instrument used."
    )
    opt_parser.add_option("f", "filtname", "", "Filter name.")
    opt_parser.add_option("ro", "relay_optics", None, "RelayOptics")

    def init_arguments(self):
        """return arguments needed for __init__ of target"""
        _, _, inst, ro = get_filt_inst_ro()
        # Use given instrument or default?
        if self.instrument:
            instrument = deserialize_object(self.instrument)
        else:
            instrument = inst
        # Use given relay optics or default?
        if self.relay_optics:
            relay_optics = deserialize_object(self.relay_optics)
        else:
            relay_optics = ro
        args = super().init_arguments()
        args["instrument"] = instrument
        args["filtname"] = self.filtname
        args["relay_optics"] = relay_optics
        return args


class SimulateRawDarkJob(SimulateRawJob):
    """job runs simulate raw dark data"""

    # Target class to make
    target = Dark

    # default template class to use
    template_default = MicadoImgCalDark

    # extension class to use
    extension = DarkExtension

    # observing block target name
    observingblock_target = "empty_sky"

    parser = copy.deepcopy(SimulateRawJob.raw_parser)
    parser.name = __name__


class SimulateRawFlatJob(SimulateOptJob):
    """job runs simulate raw flat data"""

    # Target class to make
    target = Flat

    # default template class to use
    template_default = MicadoImgCalFlat

    # extension class to use
    extension = FlatExtension

    # observing block target name
    observingblock_target = "flatlamp"

    parser = copy.deepcopy(SimulateOptJob.opt_parser)
    parser.name = __name__
